from setuptools import setup

setup(name='redshift-pandas',
      version='0.1.1',
      description='Redshift functions written in Python',
      url='https://github.com/thibaulthe/redshift-pandas',
      author='Thibault He',
      author_email='thibault.he@gmail.com',
      license='',
      packages=['redshiftpandas'],
      zip_safe=False)